﻿/* Мною, Константином aka KilkennyCat, 05 июля 2020 года создано сиё 
 * на основе "Описание формата TopoR PCB версия 1.2.0 Апрель 2017 г.".
 * k@kilkennycat.pro
 * http://kilkennycat.ru  http://kilkennycat.pro
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml.Serialization;

namespace TopoR_PCB_Classes
{
    /// <summary>
    /// Раздел «Слои». (Обязательный раздел)
    /// </summary>
    public class Layers
    {
        /// <summary>
        /// Описание слоя.
        /// </summary>
        public class Layer
        {
            /// <summary>
            /// Имя объекта или ссылка на именованный объект.
            /// </summary>
            [XmlAttribute("name")]
            public string _name;

            /// <summary>
            /// Тип слоя. Значение по умолчанию – Signal.
            /// </summary>
            [XmlAttribute("type")]
            public layer_type _type;

            /// <summary>
            /// Параметр слоя: слой содержит очертания компонентов. 
            /// </summary>
            /// <remarks>! Для сигнальных, опорных, диэлектрических и документирующих слоёв параметр compsOutline отсутствует.</remarks>
            [XmlAttribute("compsOutline")]
            public Bool _compsOutline;

            [XmlIgnore]
            public bool _compsOutlineSpecified { get { return _type == layer_type.Assy; } }

            /// <summary>
            /// Параметр слоя: толщина.
            /// </summary>
            /// <remarks>! Для документирующих слоёв и слоёв с типом Assy параметр thickness отсутствует.</remarks>
            [XmlAttribute("thickness", DataType = "float")]
            public float _thickness;

            [XmlIgnore]
            public bool _thicknessSpecified { get { return _type != layer_type.Assy; } }



            /*****************************************************************
             * Здесь находятся функции для работы с элементами класса Layer. *
             * Они не являются частью формата TopoR PCB.                     *
             * ***************************************************************/
            public Layer()
            {

            }

            public Layer(string name, layer_type type, Bool compsOutline, float thickness)
            {
                _name = name;
                _type = type;
                _compsOutline = compsOutline;
                _thickness = thickness;
            }

            public override string ToString()
            {
                return _name;
            }
            /*****************************************************************/
        }

        /// <summary>
        /// Версия раздела.
        /// </summary>
        [XmlAttribute("version")]
        public string _version;

        /// <summary>
        /// Описание слоёв в стеке. Порядок описания должен соответствовать порядку слоёв в стеке.
        /// </summary>
        [XmlArray("StackUpLayers")]
        [XmlArrayItem("Layer")]
        public List<Layer> _StackUpLayers;
        public bool ShouldSerialize_StackUpLayers()
        {
            return _StackUpLayers?.Count > 0;
        }
        /// <summary>
        /// Описание слоёв вне стека.
        /// </summary>
        [XmlArray("UnStackLayers")]
        [XmlArrayItem("Layer")]
        public List<Layer> _UnStackLayers;
        public bool ShouldSerialize_UnStackLayers()
        {
            return _UnStackLayers?.Count > 0;
        }


        /******************************************************************
         * Здесь находятся функции для работы с элементами класса Layers. *
         * Они не являются частью формата TopoR PCB.                      *
         * ****************************************************************/
        /// <summary>
        /// Проверяет существование слоя, на который ссылается ссылка
        /// </summary>
        /// <param name="lref">Ссылка на слой</param>
        /// <returns>true, если слой существует</returns>
        public bool LayerStackUpContains(LayerRef lref)
        {
            return _StackUpLayers?.Where(r => r._name == lref._ReferenceName).Count() > 0;
        }
        /// <summary>
        /// Проверяет существование слоя, на который ссылается ссылка
        /// </summary>
        /// <param name="lref">Ссылка на слой</param>
        /// <returns>true, если слой существует</returns>
        public bool LayerUnStackContain(LayerRef lref)
        {
            return _UnStackLayers?.Where(r => r._name == lref._ReferenceName).Count() > 0;
        }
        /******************************************************************/
    }
}
